'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var reactstrap = require('reactstrap');
var btComponents = require('@alejandro.devop/bt-components');
var styled = require('styled-components');
var Skeleton = require('react-loading-skeleton');
var _ = require('lodash');
var Animation = require('react-reveal/Slide');
var ReactJsonPrint = require('react-json-print');
var reactJsonApiClient = require('@alejandro.devop/react-json-api-client');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var styled__default = /*#__PURE__*/_interopDefaultLegacy(styled);
var Skeleton__default = /*#__PURE__*/_interopDefaultLegacy(Skeleton);
var ___default = /*#__PURE__*/_interopDefaultLegacy(_);
var Animation__default = /*#__PURE__*/_interopDefaultLegacy(Animation);
var ReactJsonPrint__default = /*#__PURE__*/_interopDefaultLegacy(ReactJsonPrint);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __makeTemplateObject(cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
}

var GridContext = React__default['default'].createContext({
    headers: {},
    fields: {},
    headerNames: [],
    sorts: {},
    sortNames: [],
    onChangeSort: function () { return null; },
    data: [],
    gridId: 'grid',
    allowSelect: false,
    clearFilters: function () { return null; },
    resetSorts: function () { return null; },
    viewFilters: false,
    toggleFilters: function () { return null; },
    onChangeFilter: function () { return null; },
    filtersConfig: [],
    onChangeRecordsPerpage: function () { return null; },
    filterValues: {},
    toggleSettings: function () { return null; },
    openSettings: false,
    toggleVisibility: function () { return null; },
    meta: {
        totalRecords: 0,
        totalPages: 0,
        currentPage: 0,
        nextPage: 0,
        prevPage: 0,
        recordsPerPage: 0
    },
    applyGoToPage: function () { return null; },
    selectedRecords: [],
    handleChangePage: function () { return null; }
});
GridContext.Provider;
var GridConsumer = GridContext.Consumer;

/**
 * Generates the necessary config for the grid headers
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param config
 * @returns UseGridHeadersReturnType
 */
var useGridHeaders = function (config) {
    var fields = config.fields, columnVisibility = config.columnVisibility;
    var headerNames = Object.keys(fields).filter(function (key) {
        var notReadable = fields[key].notReadable;
        if (columnVisibility && columnVisibility[key] === false) {
            return false;
        }
        return !notReadable;
    });
    var sortKeys = headerNames.filter(function (key) {
        var sort = fields[key].sort;
        return sort;
    });
    return {
        sortKeys: sortKeys,
        headerNames: headerNames
    };
};
/**
 * Generates the filter structure for the internal components.
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param fields
 * @returns
 */
var useBuildFilters = function (fields) {
    return Object.keys(fields).reduce(function (configs, currentKey) {
        var _a = fields[currentKey], _b = _a.filterOptions, filterOptions = _b === void 0 ? {} : _b, filter = _a.filter;
        if (filter) {
            var config = __assign(__assign({}, filterOptions), { key: currentKey });
            if (!config.type) {
                config.type = 'text';
            }
            configs.push(config);
        }
        return configs;
    }, []);
};
/**
 * Allows to access the GridContext
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @returns GridContextType
 */
var useGridContext = function () {
    var context = React.useContext(GridContext);
    return context;
};

/**
 * Renders the grid options
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @returns React.FC
 */
var GridOptionsRender = function () {
    var _a = useGridContext(), toggleFilters = _a.toggleFilters, textProcessor = _a.textProcessor, toggleSettings = _a.toggleSettings, viewFilters = _a.viewFilters;
    var label = viewFilters ? 'closeFilters' : 'advancedSearch';
    return (React__default['default'].createElement("div", { className: "flex flex-row" },
        React__default['default'].createElement("div", { className: "flex-1" },
            React__default['default'].createElement("div", { className: "btn-group" },
                React__default['default'].createElement("button", { className: "btn btn-white", onClick: toggleFilters },
                    React__default['default'].createElement(btComponents.Icon, { type: viewFilters ? 'times' : 'search' }),
                    React__default['default'].createElement("span", { className: "ml-2" }, textProcessor ? textProcessor(label) : label)))),
        React__default['default'].createElement("div", null,
            React__default['default'].createElement(btComponents.IconButton, { icon: "cogs", onClick: toggleSettings }))));
};

var ActionLink = styled__default['default'].a(templateObject_1$5 || (templateObject_1$5 = __makeTemplateObject(["\n    margin: 10px 10px;\n"], ["\n    margin: 10px 10px;\n"])));
var ActionsCol = styled__default['default'].th(templateObject_2$3 || (templateObject_2$3 = __makeTemplateObject(["\n    width: 1%;\n    white-space: nowrap;\n"], ["\n    width: 1%;\n    white-space: nowrap;\n"])));
var templateObject_1$5, templateObject_2$3;

/**
 * Renders the grid options per row
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param props
 * @returns
 */
var ActionsRender = function (_a) {
    var actions = _a.actions, rowRecord = _a.rowRecord;
    var onAcctionCalled = useGridContext().onAcctionCalled;
    return (React__default['default'].createElement(ActionsCol, null, actions.map(function (action, key) { return (React__default['default'].createElement(ActionLink, { key: "action-key-" + key, onClick: function () { return (onAcctionCalled ? onAcctionCalled(action.id, rowRecord) : null); } },
        React__default['default'].createElement("i", { className: "fa fa-" + action.icon }))); })));
};

var UndefinedText = styled__default['default'].span(templateObject_1$4 || (templateObject_1$4 = __makeTemplateObject(["\n    opacity: 0.3;\n    font-size: 12px;\n"], ["\n    opacity: 0.3;\n    font-size: 12px;\n"])));
var ColWrapper = styled__default['default'].td(templateObject_2$2 || (templateObject_2$2 = __makeTemplateObject(["\n    width: 30px;\n    text-align: center;\n"], ["\n    width: 30px;\n    text-align: center;\n"])));
var Input$2 = styled__default['default'].input(templateObject_3$2 || (templateObject_3$2 = __makeTemplateObject(["\n    margin-left: 0;\n"], ["\n    margin-left: 0;\n"])));
var templateObject_1$4, templateObject_2$2, templateObject_3$2;

/**
 * Renders the control for row selection
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @returns React.FC
 */
var ColSelectControl = function () {
    return (React__default['default'].createElement(ColWrapper, null,
        React__default['default'].createElement(Input$2, { className: "form-check-input", type: "checkbox", id: "checkbox1" })));
};

/**
 * Renders the data for the grid body
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @returns React.FC
 */
var DataRenderer = function () {
    var _a = useGridContext(), actions = _a.actions, data = _a.data, headerNames = _a.headerNames, allowSelect = _a.allowSelect, fields = _a.fields, textProcessor = _a.textProcessor;
    var processText = function (text) { return (textProcessor ? textProcessor(text) : text); };
    var resolveDotNotation = function (value, rowValue) {
        if (value.includes('.')) {
            var parts = value.split('.');
            var reducedValue = parts.reduce(function (newVal, currentPart) {
                if (typeof newVal === 'object') {
                    newVal = newVal[currentPart];
                }
                return newVal;
            }, rowValue);
            return reducedValue;
        }
        return value;
    };
    var valueResolver = function (_a, rowValue) {
        var value = _a.value;
        if (typeof value === 'string') {
            return resolveDotNotation(value, rowValue);
        }
        else if (typeof value === 'function') {
            var resolvedValue = value(rowValue);
            return resolvedValue;
            // return typeof resolvedValue == 'string' ? resolvedValue : 'invalid value'
        }
        return '';
    };
    return (React__default['default'].createElement("tbody", null, data &&
        Array.isArray(data) &&
        (data === null || data === void 0 ? void 0 : data.map(function (rowData, rowId) { return (React__default['default'].createElement("tr", { key: "row-" + rowId },
            allowSelect && React__default['default'].createElement(ColSelectControl, null),
            headerNames.map(function (key) {
                var colConfig = fields[key];
                var resolvedValue = valueResolver(colConfig, rowData);
                return (React__default['default'].createElement("td", { key: "col-row-" + key }, resolvedValue || (React__default['default'].createElement(UndefinedText, null,
                    "( ",
                    processText('undefinedValue'),
                    " )"))));
            }),
            actions && React__default['default'].createElement(ActionsRender, { actions: actions, rowRecord: rowData }))); }))));
};

var HeaderCol = styled__default['default'].th(templateObject_1$3 || (templateObject_1$3 = __makeTemplateObject(["\n    cursor: pointer;\n"], ["\n    cursor: pointer;\n"])));
var ControlCol = styled__default['default'].th(templateObject_2$1 || (templateObject_2$1 = __makeTemplateObject(["\n    width: 30px;\n    text-align: center;\n"], ["\n    width: 30px;\n    text-align: center;\n"])));
var HeaderContainer = styled__default['default'].div(templateObject_3$1 || (templateObject_3$1 = __makeTemplateObject(["\n    display: flex;\n    flex-direction: 'row';\n"], ["\n    display: flex;\n    flex-direction: 'row';\n"])));
var HeaderText = styled__default['default'].span(templateObject_4 || (templateObject_4 = __makeTemplateObject(["\n    flex: 1;\n"], ["\n    flex: 1;\n"])));
var HeaderSortIcon = styled__default['default'](btComponents.Icon)(templateObject_5 || (templateObject_5 = __makeTemplateObject(["\n    opacity: 0.4;\n"], ["\n    opacity: 0.4;\n"])));
var Input$1 = styled__default['default'].input(templateObject_6 || (templateObject_6 = __makeTemplateObject(["\n    margin-left: 15px;\n    position: relative;\n"], ["\n    margin-left: 15px;\n    position: relative;\n"])));
var templateObject_1$3, templateObject_2$1, templateObject_3$1, templateObject_4, templateObject_5, templateObject_6;

/**
 * Renders a single column for the header, renders the icon if the header requires sort
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param props
 * @returns
 */
var HeaderColRenderer = function (_a) {
    var name = _a.name, withSort = _a.withSort, sortValue = _a.sortValue, headerExtraConfig = _a.headerExtraConfig;
    var _b = headerExtraConfig || {}, label = _b.label, hide = _b.hide;
    var _c = useGridContext(), headerTextProcessor = _c.headerTextProcessor, onChangeSort = _c.onChangeSort;
    var getIcon = function () {
        if (sortValue === 1) {
            return 'sort-up';
        }
        else if (sortValue === -1) {
            return 'sort-down';
        }
        return 'sort';
    };
    var props = withSort
        ? { onClick: function () { return onChangeSort(name); } }
        : { style: { cursor: 'default' } };
    return (React__default['default'].createElement(HeaderCol, __assign({}, props),
        React__default['default'].createElement(HeaderContainer, null,
            !hide && (React__default['default'].createElement(HeaderText, null, headerTextProcessor ? headerTextProcessor(label || name) : label || name)),
            withSort && React__default['default'].createElement(HeaderSortIcon, { type: getIcon(), disabled: sortValue === 0 }))));
};

/**
 *  Renders the headers for the grid, there is nothing special, except for the
 * validation, if the grid includes actions, this renderer add an extra column at the end.
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param props
 * @returns
 */
var HeaderRenderer = function (_a) {
    var withActions = _a.withActions;
    var _b = useGridContext(), gridId = _b.gridId, allowSelect = _b.allowSelect, headerNames = _b.headerNames, sortNames = _b.sortNames, sorts = _b.sorts, headers = _b.headers;
    return (React__default['default'].createElement("thead", null,
        React__default['default'].createElement("tr", null,
            allowSelect && (React__default['default'].createElement(ControlCol, null,
                React__default['default'].createElement(Input$1, { className: "form-check-input", type: "checkbox", id: "checkbox1" }))),
            headerNames.map(function (headerName, key) { return (React__default['default'].createElement(HeaderColRenderer, { key: gridId + "-" + key, name: headerName, headerExtraConfig: headers[headerName], withSort: sortNames.includes(headerName), sortValue: sorts[headerName] })); }),
            withActions && React__default['default'].createElement("th", null, "\u00A0"))));
};

/**
 * Renders The loader for the grid while it's feching data
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param props
 * @returns
 */
var GridLoader = function (_a) {
    var cols = _a.cols, rows = _a.rows;
    return (React__default['default'].createElement("tbody", null, ___default['default'].times(rows, function (rowId) { return (React__default['default'].createElement("tr", { key: "skeleton-row-" + rowId }, ___default['default'].times(cols, function (key) { return (React__default['default'].createElement("td", { key: "skeleton-col-" + rowId + "-" + key },
        React__default['default'].createElement(Skeleton__default['default'], null))); }))); })));
};
GridLoader.defaultProps = {
    cols: 10,
    rows: 10
};

var Paragraph$2 = styled__default['default'].p(templateObject_1$2 || (templateObject_1$2 = __makeTemplateObject(["\n    margin-bottom: 0;\n    padding-bottom: 0;\n    font-size: 12px;\n    margin-right: 10px;\n"], ["\n    margin-bottom: 0;\n    padding-bottom: 0;\n    font-size: 12px;\n    margin-right: 10px;\n"])));
var templateObject_1$2;

/**
 * Renders the applied filters
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @returns
 */
var AppliedFilters = function () {
    var _a = useGridContext(), textProcessor = _a.textProcessor, filterValues = _a.filterValues, displayFiltersFormatter = _a.displayFiltersFormatter, headerTextProcessor = _a.headerTextProcessor, onChangeFilter = _a.onChangeFilter;
    var processText = function (key) { return (textProcessor ? textProcessor(key) : key); };
    var reducedFilters = Object.keys(filterValues).reduce(function (filters, currentKey) {
        if (filterValues[currentKey]) {
            filters.push({ label: currentKey, value: filterValues[currentKey] });
        }
        return filters;
    }, []);
    var processHeaderText = function (text) {
        return headerTextProcessor ? headerTextProcessor(text) : text;
    };
    var handleClearFilter = function (key) {
        onChangeFilter(key, '');
    };
    if (displayFiltersFormatter) {
        reducedFilters = displayFiltersFormatter(reducedFilters);
    }
    if (reducedFilters.length === 0) {
        return null;
    }
    return (React__default['default'].createElement("div", { className: "flex flex-row items-center" },
        React__default['default'].createElement(Paragraph$2, null, processText('appliedFilters')),
        reducedFilters.map(function (filter, key) { return (React__default['default'].createElement("span", { key: "filter-applied-" + key, className: "badge bg-indigo flex items-center flex-row mr-2" }, processHeaderText(filter.label) + ": " + filter.value,
            ' ',
            React__default['default'].createElement(btComponents.IconButton, { onClick: function () { return handleClearFilter(filter.label); }, icon: "times", size: "xs" }))); })));
};

/**
 * This map is used to configure the grid filter fields
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 */
var fieldsMap = {
    checkbox: btComponents.CheckBoxField,
    date: btComponents.DateField,
    dropdown: btComponents.DropDownField,
    email: btComponents.EmailField,
    file: btComponents.FilePickerField,
    password: btComponents.PasswordField,
    radio: btComponents.RadioField,
    select: btComponents.SelectField,
    textarea: btComponents.TextArea,
    text: btComponents.TextField,
    time: btComponents.TimeField
};

// let timerForInputChange: any
/**
 * Renders a single filter item and resolves it depending on the type
 * @param props
 * @returns
 */
var RenderFilter = function (_a) {
    var filterConfig = _a.filterConfig;
    var _b = useGridContext(), headerTextProcessor = _b.headerTextProcessor, filterValues = _b.filterValues, onChangeFilter = _b.onChangeFilter;
    var processText = function (text) { return (headerTextProcessor ? headerTextProcessor(text) : text); };
    var label = filterConfig.label, key = filterConfig.key, name = filterConfig.name, type = filterConfig.type, selectOptions = filterConfig.selectOptions, preProcessFilterValue = filterConfig.preProcessFilterValue;
    var FieldComponent = fieldsMap.text;
    if (type) {
        FieldComponent = fieldsMap[type] || fieldsMap.text;
    }
    var keyToChange = name || key;
    var currentValue = filterValues[keyToChange];
    var handleChange = function (_a) {
        var target = _a.target;
        var name = target.name, value = target.value;
        onChangeFilter(name, preProcessFilterValue ? preProcessFilterValue(value) : value);
    };
    var componentProps = {
        placeholder: processText(key),
        label: label,
        name: keyToChange,
        onChange: handleChange,
        value: currentValue
    };
    /**
     * If the type is select then there are some special props that maybe we will need to pre-process
     */
    if (type === 'select') {
        var _c = selectOptions || {}, advanced = _c.advanced, ajax = _c.ajax, columnsMapping = _c.columnsMapping, requestConfig = _c.requestConfig, disableGroup = _c.disableGroup, empty = _c.empty, multiple = _c.multiple, url = _c.url, value = _c.value;
        componentProps = __assign(__assign({}, componentProps), { requestConfig: requestConfig,
            advanced: advanced,
            ajax: ajax,
            columnsMapping: columnsMapping,
            disableGroup: disableGroup,
            empty: empty,
            multiple: multiple,
            url: url, value: value });
    }
    return (React__default['default'].createElement(reactstrap.Col, { xs: "12", sm: "6", md: "3", className: "mt-2" },
        React__default['default'].createElement("div", { className: " input-group-sm" },
            React__default['default'].createElement(FieldComponent, __assign({}, componentProps)))));
};

/**
 * Renders structure for the filters used in the grid
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param props
 * @returns
 */
var GridSearch = function (_a) {
    var visible = _a.visible;
    var _b = useGridContext(), toggleFilters = _b.toggleFilters, textProcessor = _b.textProcessor, filtersConfig = _b.filtersConfig, clearFilters = _b.clearFilters;
    var processText = function (text) { return (textProcessor ? textProcessor(text) : text); };
    return (React__default['default'].createElement(React__default['default'].Fragment, null,
        React__default['default'].createElement(reactstrap.Container, { fluid: true, className: "mt-2" },
            React__default['default'].createElement(reactstrap.Row, null,
                React__default['default'].createElement(reactstrap.Col, { xs: "12" },
                    React__default['default'].createElement(AppliedFilters, null)))),
        visible && (React__default['default'].createElement(Animation__default['default'], { right: true, collapse: true, when: visible === true },
            React__default['default'].createElement(reactstrap.Container, { className: "mb-4 bg-white", fluid: true },
                React__default['default'].createElement(reactstrap.Row, null, filtersConfig.map(function (filter, key) {
                    return React__default['default'].createElement(RenderFilter, { key: "filter-" + key, filterConfig: filter });
                })),
                React__default['default'].createElement(reactstrap.Row, null,
                    React__default['default'].createElement(reactstrap.Col, { xs: "12", className: "mt-4" },
                        React__default['default'].createElement("button", { type: "button", className: "btn btn-sm btn-primary mr-2", onClick: clearFilters },
                            React__default['default'].createElement(btComponents.Icon, { type: "eraser" }),
                            React__default['default'].createElement("span", { className: "ml-2" }, processText('clearFilters'))),
                        React__default['default'].createElement("button", { type: "button", className: "btn btn-sm btn-white", onClick: toggleFilters },
                            React__default['default'].createElement(btComponents.Icon, { type: "times" }),
                            React__default['default'].createElement("span", { className: "ml-2" }, processText('closeFilters'))))))))));
};

var Paragraph$1 = styled__default['default'].p(templateObject_1$1 || (templateObject_1$1 = __makeTemplateObject(["\n    margin-bottom: 2px;\n"], ["\n    margin-bottom: 2px;\n"])));
var Input = styled__default['default'].input(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n    width: 120px;\n    text-align: center;\n"], ["\n    width: 120px;\n    text-align: center;\n"])));
var Pagination = styled__default['default'](reactstrap.Pagination)(templateObject_3 || (templateObject_3 = __makeTemplateObject(["\n    & ul {\n        margin-bottom: 7px;\n    }\n"], ["\n    & ul {\n        margin-bottom: 7px;\n    }\n"])));
var templateObject_1$1, templateObject_2, templateObject_3;

/**
 * Function to check and trigger a callback only when the Enter key is pressed
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param e
 * @param callBack
 */
var listenForEnterKey = function (e, callBack) {
    var key = e.key;
    if (key === 'Enter') {
        callBack();
    }
};

/**
 * Timer used to delay the apply action for the goToPage button
 */
var timerForApplyGoTo = null;
/**
 * Only handles the pagination render structure, has managed states for the inputs, but
 * it transfer the entered data right to it's parent controller.
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param props
 * @returns
 */
var GridPaginationDisplay = function (_a) {
    var pagesToRender = _a.pagesToRender, disableGoTo = _a.disableGoTo, disablePager = _a.disablePager, disableStepper = _a.disableStepper, goToFirst = _a.goToFirst, goToLast = _a.goToLast, handleNext = _a.handleNext, handlePrev = _a.handlePrev, recordsPerPageControl = _a.recordsPerPageControl, onChangeRecordsPerPage = _a.onChangeRecordsPerPage, onApplyRecordsPerPage = _a.onApplyRecordsPerPage, activePage = _a.activePage, limit = _a.limit, handleClickOnNumber = _a.handleClickOnNumber, shouldSkipPages = _a.shouldSkipPages, middlePoint = _a.middlePoint, handleApplyGoTo = _a.handleApplyGoTo, _b = _a.step, step = _b === void 0 ? 10 : _b;
    var _c = useGridContext(), textProcessor = _c.textProcessor, meta = _c.meta;
    var totalRecords = meta.totalRecords, totalPages = meta.totalPages;
    var _d = React.useState(''), currentGoToPage = _d[0], setCurrentGoToPage = _d[1];
    var _e = React.useState(false), enabledEnterStroke = _e[0], setEnableEnterStroke = _e[1];
    var processText = function (key) { return (textProcessor ? textProcessor(key) : key); };
    /**
     * Controls the value change for the records per page state
     * @param event
     */
    var handleChangeRecordsPerpage = function (_a) {
        var target = _a.target;
        var value = target.value;
        if (!isNaN(value)) {
            onChangeRecordsPerPage(parseInt(value, 10));
        }
    };
    /**
     * Control for the add records per page button (When the user presses the add button)
     */
    var handleUpRecordsPerPage = function () {
        var newValue = recordsPerPageControl + step;
        if (newValue > 100) {
            newValue = 100;
        }
        onChangeRecordsPerPage(newValue);
    };
    /**
     * Control for the substract records per apge button (When the user presses the substract button)
     */
    var handleDownRecordsPerPage = function () {
        var newValue = recordsPerPageControl - step;
        if (newValue < step) {
            newValue = step;
        }
        onChangeRecordsPerPage(newValue);
    };
    /**
     * Controls the value change for the input "go to page"
     */
    var handleChangeGoTo = function (_a) {
        var target = _a.target;
        var value = target.value;
        // If the user enters a letter or a non number we dont want to to proceed with the go to action
        if (!isNaN(value) && parseInt(value, 10) >= 1) {
            setCurrentGoToPage(value);
            clearTimeout(timerForApplyGoTo);
            // this delays allows to stop the server request call action.
            timerForApplyGoTo = setTimeout(function () {
                if (parseInt(value, 10) > totalPages) {
                    setCurrentGoToPage(totalPages);
                }
                setEnableEnterStroke(true);
            }, 200);
        }
        else {
            setCurrentGoToPage('');
        }
    };
    /**
     * A totally different action for the apply (Go to page), allows us to reuse the code
     * and attach this behaviour to the enter action
     */
    var handleApply = function () {
        handleApplyGoTo(parseInt(currentGoToPage, 10));
        setCurrentGoToPage('');
    };
    React.useEffect(function () {
        // We want to make sure we set free the resources for this timer.
        return function () { return clearTimeout(timerForApplyGoTo); };
    }, []);
    return (React__default['default'].createElement("div", { className: "flex flex-1 items-end justify-end" },
        React__default['default'].createElement("div", { className: " mt-6 mb-0 flex flex-column items-start flex-1" },
            React__default['default'].createElement(Paragraph$1, null,
                React__default['default'].createElement(btComponents.Icon, { type: "sort-amount-up" }),
                React__default['default'].createElement("span", { className: "ml-2" }, textProcessor
                    ? textProcessor('totalRecords', {
                        key: 'records',
                        value: totalRecords
                    })
                    : 'records')),
            !disablePager && (React__default['default'].createElement(Pagination, { className: "mr-12 mb-0 pb-0", size: "sm" },
                activePage > 1 && (React__default['default'].createElement(React__default['default'].Fragment, null,
                    React__default['default'].createElement(reactstrap.PaginationItem, { disabled: activePage === 1 },
                        React__default['default'].createElement(reactstrap.PaginationLink, { onClick: goToFirst, href: "#" }, processText('first'))),
                    React__default['default'].createElement(reactstrap.PaginationItem, { disabled: activePage === 1 },
                        React__default['default'].createElement(reactstrap.PaginationLink, { onClick: handlePrev, href: "#" }, processText('previous'))))),
                pagesToRender.map(function (page) { return (React__default['default'].createElement(reactstrap.PaginationItem, { key: "pagination-item-" + page, active: activePage === page },
                    React__default['default'].createElement(reactstrap.PaginationLink, { href: "#", onClick: function (e) { return handleClickOnNumber(e, page); } }, page))); }),
                shouldSkipPages && activePage < totalPages - (limit - middlePoint) && (React__default['default'].createElement(React__default['default'].Fragment, null,
                    React__default['default'].createElement(reactstrap.PaginationItem, { disabled: true },
                        React__default['default'].createElement(reactstrap.PaginationLink, { href: "#" }, "...")),
                    React__default['default'].createElement(reactstrap.PaginationItem, null,
                        React__default['default'].createElement(reactstrap.PaginationLink, { onClick: function (e) { return handleClickOnNumber(e, totalPages); }, href: "#" }, totalPages)))),
                activePage < totalPages && (React__default['default'].createElement(React__default['default'].Fragment, null,
                    React__default['default'].createElement(reactstrap.PaginationItem, { disabled: activePage === totalPages },
                        React__default['default'].createElement(reactstrap.PaginationLink, { onClick: handleNext, href: "#" }, processText('next'))),
                    React__default['default'].createElement(reactstrap.PaginationItem, { disabled: activePage === totalPages },
                        React__default['default'].createElement(reactstrap.PaginationLink, { onClick: goToLast, href: "#" }, processText('last')))))))),
        !disableStepper && (React__default['default'].createElement("div", { className: "mb-2 mr-8" },
            React__default['default'].createElement(Paragraph$1, null, textProcessor ? textProcessor('recordsPerPage') : 'Records per page'),
            React__default['default'].createElement("div", { className: "input-group input-group-sm" },
                React__default['default'].createElement("div", { className: "input-group-prepend" },
                    React__default['default'].createElement("button", { className: "btn btn-white btn-sm", onClick: function () { return handleDownRecordsPerPage(); } },
                        React__default['default'].createElement(btComponents.Icon, { type: "chevron-down" }))),
                React__default['default'].createElement(Input, { type: "number", className: "form-control", placeholder: "Records per page", onChange: handleChangeRecordsPerpage, onKeyUp: function (e) { return listenForEnterKey(e, function () { }); }, readOnly: true, max: 100, value: recordsPerPageControl }),
                React__default['default'].createElement("div", { className: "input-group-append" },
                    React__default['default'].createElement("button", { className: "btn btn-white btn-sm", onClick: function () { return handleUpRecordsPerPage(); } },
                        React__default['default'].createElement(btComponents.Icon, { type: "chevron-up" }))),
                React__default['default'].createElement("div", { className: "input-group-append" },
                    React__default['default'].createElement("button", { className: "btn btn-white btn-sm", onClick: function () { return onApplyRecordsPerPage(); } },
                        React__default['default'].createElement("span", { className: "mr-2" }, textProcessor ? textProcessor('apply') : 'Apply'),
                        React__default['default'].createElement(btComponents.Icon, { type: "check" })))))),
        !disableGoTo && (React__default['default'].createElement("div", { className: "mb-2" },
            React__default['default'].createElement(Paragraph$1, null, textProcessor
                ? textProcessor('totalPages', { key: 'pages', value: totalPages })
                : 'Total pages'),
            React__default['default'].createElement("div", { className: "input-group input-group-sm" },
                React__default['default'].createElement(Input, { type: "number", className: "form-control", placeholder: textProcessor ? textProcessor('gotoPage') : 'Go to page', value: currentGoToPage, onChange: handleChangeGoTo, onKeyUp: function (e) {
                        return listenForEnterKey(e, function () {
                            if (enabledEnterStroke) {
                                handleApply();
                            }
                        });
                    }, onKeyDown: function (e) {
                        // Using this we restrict the enter press util the
                        // timeout validates if the entered value is correct
                        if (e.key !== 'Enter') {
                            setEnableEnterStroke(false);
                        }
                    } }),
                React__default['default'].createElement("div", { className: "input-group-append" },
                    React__default['default'].createElement("button", { className: "btn btn-primary btn-sm", disabled: _.isEmpty(currentGoToPage) || !enabledEnterStroke, onClick: handleApply }, textProcessor ? textProcessor('go') : 'Go')))))));
};
GridPaginationDisplay.defaultProps = {
    step: 10
};

/**
 * Controller for the pagination, it provides all the logic for the pagination renderer
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @returns React.FC
 */
var GridPaginationRender = function (_a) {
    var disableGoTo = _a.disableGoTo, disableStepper = _a.disableStepper, disablePager = _a.disablePager;
    var _b = useGridContext(), meta = _b.meta, maxPagesToDisplay = _b.maxPagesToDisplay, handleChangePage = _b.handleChangePage, applyGoToPage = _b.applyGoToPage, onChangeRecordsPerpage = _b.onChangeRecordsPerpage;
    var totalPages = meta.totalPages, currentPage = meta.currentPage, recordsPerPage = meta.recordsPerPage;
    var _c = React.useState(currentPage), activePage = _c[0], setPage = _c[1];
    var _d = React.useState(recordsPerPage), recordsPerPageControl = _d[0], setRecordsPerpage = _d[1];
    var shouldSkipPages = maxPagesToDisplay && totalPages > maxPagesToDisplay;
    // ToDo: Move this to a prop
    var limit = 10;
    /**
     * The following logic allows to control the displacement for the pagination cursor
     * @note Can be improved
     * Begin:
     */
    var middlePoint = Math.ceil(limit / 2) + 1;
    var beginPoint = 0;
    var endPoint = limit;
    var rightIncrement = 0;
    var leftIncrement = 0;
    if (activePage > middlePoint) {
        var diff = activePage - middlePoint;
        if (activePage <= totalPages - (limit - middlePoint)) {
            leftIncrement = diff;
        }
        else {
            leftIncrement = totalPages - limit;
        }
        rightIncrement = diff;
    }
    var pagesMap = Array.from({ length: totalPages }, function (_, i) { return i + 1; });
    var pagesToRender = pagesMap.slice(beginPoint + leftIncrement, endPoint + rightIncrement);
    /**
     * End
     */
    /**
     * Handles the increment when the user presses next
     */
    var handleNext = function (event) {
        event.preventDefault();
        var nextValue = activePage;
        if (activePage < totalPages) {
            nextValue = activePage + 1;
            setPage(nextValue);
        }
        if (nextValue !== activePage) {
            handleChangePage(nextValue);
        }
    };
    /**
     * Handles the decrement when the user presses substract
     */
    var handlePrev = function (event) {
        event.preventDefault();
        var nextValue = activePage;
        if (activePage > 1) {
            nextValue = activePage - 1;
            setPage(nextValue);
        }
        if (nextValue !== activePage) {
            handleChangePage(nextValue);
        }
    };
    /**
     * Allows to go the first page
     * @note always will be 1
     * @param event
     */
    var goToFirst = function (event) {
        event.preventDefault();
        setPage(1);
        if (activePage !== 1) {
            handleChangePage(1);
        }
    };
    /**
     * Allows to go the last page (provided by the api or the smart grid)
     * @param event
     */
    var goToLast = function (event) {
        event.preventDefault();
        setPage(totalPages);
        if (activePage !== totalPages) {
            handleChangePage(totalPages);
        }
    };
    /**
     * Handles page change when the user presses the displayed page numbers
     * @param event
     * @param num
     */
    var handleClickOnNumber = function (event, num) {
        event.preventDefault();
        setPage(num);
        handleChangePage(num);
    };
    /**
     * Handles the change for the records per page value, but it's only used for the display value on the input
     */
    var handleChangeRecordsPerpage = function (newValue) {
        setRecordsPerpage(newValue);
    };
    /**
     * Sends the displayed value to the official records per page state, it automatically trigers a new
     * server request.
     */
    var onApplyRecordsPerPage = function () {
        setPage(1);
        onChangeRecordsPerpage(recordsPerPageControl);
    };
    /**
     * Sends the displayed value to the official currentPage state, it automatically trigers a new
     * server request
     * @param n new value for the current page state.
     */
    var handleApplyGoTo = function (n) {
        setPage(n);
        applyGoToPage(n);
    };
    return (React__default['default'].createElement(GridPaginationDisplay, { disableGoTo: disableGoTo, disableStepper: disableStepper, disablePager: disablePager, handleApplyGoTo: handleApplyGoTo, activePage: activePage, onChangeRecordsPerPage: handleChangeRecordsPerpage, recordsPerPageControl: recordsPerPageControl, onApplyRecordsPerPage: onApplyRecordsPerPage, goToFirst: goToFirst, goToLast: goToLast, handleClickOnNumber: handleClickOnNumber, handleNext: handleNext, handlePrev: handlePrev, limit: limit, middlePoint: middlePoint, pagesToRender: pagesToRender, shouldSkipPages: shouldSkipPages }));
};

/**
 * Renders debug options for the grid
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @returns React.FC
 */
var GridDebugger = function () {
    var _a = useGridContext(), gridId = _a.gridId, headers = _a.headers, meta = _a.meta, filtersConfig = _a.filtersConfig, filterValues = _a.filterValues, sorts = _a.sorts, loading = _a.loading, viewFilters = _a.viewFilters;
    return (React__default['default'].createElement(ReactJsonPrint__default['default'], { expanded: true, dataObject: {
            filterValues: filterValues,
            filtersConfig: filtersConfig,
            gridId: gridId,
            headers: headers,
            loading: loading,
            meta: meta,
            sorts: sorts,
            viewFilters: viewFilters
        } }));
};

var Paragraph = styled__default['default'].p(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n    margin-bottom: 0;\n    padding-bottom: 0;\n    font-size: 12px;\n    margin-right: 10px;\n    margin-left: 15px;\n"], ["\n    margin-bottom: 0;\n    padding-bottom: 0;\n    font-size: 12px;\n    margin-right: 10px;\n    margin-left: 15px;\n"])));
var templateObject_1;

/**
 * Renders the applied sorts for the grid
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @returns React.FC
 */
var AppliedSorts = function () {
    var _a = useGridContext(), textProcessor = _a.textProcessor, sorts = _a.sorts, headerTextProcessor = _a.headerTextProcessor, resetSorts = _a.resetSorts;
    var processHeaderText = function (text) {
        return headerTextProcessor ? headerTextProcessor(text) : text;
    };
    var processText = function (key) { return (textProcessor ? textProcessor(key) : key); };
    var appliedSorts = Object.keys(sorts).reduce(function (values, currentKey) {
        if (sorts[currentKey] !== 0) {
            values.push({
                key: currentKey,
                label: processHeaderText(currentKey),
                value: sorts[currentKey] > 0 ? 'asc' : 'desc'
            });
        }
        return values;
    }, []);
    if (appliedSorts.length === 0) {
        return null;
    }
    return (React__default['default'].createElement("div", { className: "flex flex-row items-center mt-2" },
        React__default['default'].createElement(Paragraph, null,
            processText('appliedSorts'),
            ":"),
        appliedSorts.map(function (item, key) { return (React__default['default'].createElement("span", { className: "badge bg-default flex items-center flex-row mr-2", key: "sorts-applied-" + key },
            item.label,
            ": ",
            item.value)); }),
        React__default['default'].createElement(btComponents.IconButton, { icon: "times", type: "info", size: "sm", onClick: resetSorts })));
};

/**
 * Assembles and renders the grid pieces
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @returns React.FC
 */
var GridRender = function () {
    return (React__default['default'].createElement(GridConsumer, null, function (_a) {
        var actions = _a.actions, loadingLabel = _a.loadingLabel, debug = _a.debug, loading = _a.loading, headerNames = _a.headerNames, viewFilters = _a.viewFilters;
        return (React__default['default'].createElement(React__default['default'].Fragment, null,
            React__default['default'].createElement("div", null,
                debug && React__default['default'].createElement(GridDebugger, null),
                React__default['default'].createElement(GridOptionsRender, null),
                React__default['default'].createElement(GridSearch, { visible: viewFilters }),
                React__default['default'].createElement(AppliedSorts, null),
                loading && (React__default['default'].createElement("div", { className: "flex flex-row items-center" },
                    React__default['default'].createElement(reactstrap.Spinner, { className: "mr-3", type: "grow", color: "primary", size: "sm" }),
                    loadingLabel && React__default['default'].createElement("span", { className: "text-xs" }, loadingLabel))),
                React__default['default'].createElement(GridPaginationRender, { disableGoTo: true, disableStepper: true, disablePager: true }),
                React__default['default'].createElement("table", { className: "table table-striped " },
                    React__default['default'].createElement(HeaderRenderer, { withActions: Boolean(actions) }),
                    !loading && React__default['default'].createElement(DataRenderer, null),
                    loading && (React__default['default'].createElement(GridLoader, { cols: headerNames.length + (Boolean(actions) ? 1 : 0), rows: 10 }))),
                React__default['default'].createElement(GridPaginationRender, null))));
    }));
};

var timerForInputValueChange = null;
/**
 * Renders the Smart grid component, a grid with capabilities to:
 * - Fetch data from a remote origin
 * - Handle filters
 * - Handle sorts
 * - Handle actions
 * - Pagination
 * - Advanced pagination features
 * - Displayed records control
 * Everything controlled (or configured) using JSON files or remote json files (Check the RSmartGrid Component)
 * @author Alejandro Quiroz <alejandro.devop@gmail.com>
 * @version 1.0.0
 * @param props
 * @returns
 */
var SmartGrid = function (_a) {
    var config = _a.config, headerTextProcessor = _a.headerTextProcessor, onAcctionCalled = _a.onAcctionCalled, textProcessor = _a.textProcessor;
    var displayFiltersFormatter = config.displayFiltersFormatter, _b = config.maxPagesToDisplay, maxPagesToDisplay = _b === void 0 ? 10 : _b, loadingLabel = config.loadingLabel, allowSelect = config.allowSelect, debug = config.debug, headers = config.headers, actions = config.actions, fields = config.fields, id = config.id, filtersVisible = config.filtersVisible, url = config.url, defaultFilters = config.defaultFilters, columnVisibility = config.columnVisibility;
    var initialFilters = function () {
        return filtersConfig.reduce(function (currentValue, currentItem) {
            currentValue[currentItem.key] = '';
            return currentValue;
        }, {});
    };
    var initialSorts = function () {
        return sortKeys.reduce(function (val, curr) {
            val[curr] = 0;
            return val;
        }, {});
    };
    var _c = React.useState(columnVisibility), colsVisibility = _c[0], setVisibility = _c[1];
    var _d = React.useState(false), openSettings = _d[0], setOpenSettings = _d[1];
    var _e = React.useState({
        totalRecords: 0,
        totalPages: 1,
        currentPage: 1,
        nextPage: 2,
        prevPage: 0,
        recordsPerPage: 10
    }), meta = _e[0], setMeta = _e[1];
    var filtersConfig = useBuildFilters(fields);
    var _f = React.useState(initialFilters()), filterValues = _f[0], setFilterVaues = _f[1];
    var _g = React.useState({}), filtersToApply = _g[0], setFiltersToApply = _g[1];
    var currentPage = meta.currentPage, recordsPerPage = meta.recordsPerPage;
    var _h = reactJsonApiClient.useGet(url, {
        onCompleted: function (_a) {
            var requestMeta = _a.meta;
            var page = (requestMeta || {}).page;
            setMeta(__assign(__assign({}, meta), { totalRecords: (page === null || page === void 0 ? void 0 : page.total) || meta.totalRecords, currentPage: (page === null || page === void 0 ? void 0 : page['current-page']) || meta.currentPage, totalPages: (page === null || page === void 0 ? void 0 : page['last-page']) || meta.totalPages, recordsPerPage: (page === null || page === void 0 ? void 0 : page['per-page']) || meta.recordsPerPage }));
        },
        filters: __assign(__assign({}, defaultFilters), filtersToApply),
        pageSize: recordsPerPage,
        currentPage: currentPage
    }), data = _h[0], loading = _h[1].loading;
    var selectedRecords = React.useState([])[0];
    var applyGoToPage = function (newPage) { return setMeta(__assign(__assign({}, meta), { currentPage: newPage })); };
    var _j = useGridHeaders({ fields: fields, columnVisibility: colsVisibility }), headerNames = _j.headerNames, sortKeys = _j.sortKeys;
    var _k = React.useState(Boolean(filtersVisible)), viewFilters = _k[0], setViewFilters = _k[1];
    var _l = React.useState(initialSorts()), sorts = _l[0], setSorts = _l[1];
    var toggleSettings = function () { return setOpenSettings(!openSettings); };
    var toggleFilters = function () { return setViewFilters(!viewFilters); };
    var onChangeFilter = function (key, value) {
        var _a;
        var newValues = __assign(__assign({}, filterValues), (_a = {}, _a[key] = value, _a));
        setFilterVaues(newValues);
    };
    var toggleVisibility = function (key, newValue) {
        var _a;
        return setVisibility(__assign(__assign({}, colsVisibility), (_a = {}, _a[key] = newValue, _a)));
    };
    var resetSorts = function () {
        setSorts(initialSorts());
    };
    var handleClearFilters = function () {
        setFilterVaues(initialFilters());
    };
    var handleChangePage = function (newPage) {
        setMeta(__assign(__assign({}, meta), { currentPage: newPage }));
    };
    var handleSortChange = function (key) {
        var _a;
        var currentValue = sorts[key];
        if (currentValue === 0) {
            currentValue = 1;
        }
        else if (currentValue === 1) {
            currentValue = -1;
        }
        else if (currentValue === -1) {
            currentValue = 0;
        }
        setSorts(__assign(__assign({}, sorts), (_a = {}, _a[key] = currentValue, _a)));
    };
    var onChangeRecordsPerpage = function (newValue) {
        setMeta(__assign(__assign({}, meta), { currentPage: 1, recordsPerPage: newValue }));
    };
    React.useEffect(function () {
        if (JSON.stringify(filterValues) !== JSON.stringify(filtersToApply)) {
            clearTimeout(timerForInputValueChange);
            timerForInputValueChange = setTimeout(function () {
                setFiltersToApply(Object.keys(filterValues).reduce(function (fta, currentKey) {
                    if (filterValues[currentKey] !== null && filterValues[currentKey] !== '') {
                        fta[currentKey] = filterValues[currentKey];
                    }
                    return fta;
                }, {}));
            }, 1000);
        }
        return function () {
            clearTimeout(timerForInputValueChange);
        };
    }, [filterValues, filtersToApply]);
    return (React__default['default'].createElement(GridContext.Provider, { value: {
            onChangeRecordsPerpage: onChangeRecordsPerpage,
            // applyRecordsPerPage,
            applyGoToPage: applyGoToPage,
            selectedRecords: selectedRecords,
            headerNames: headerNames,
            headers: headers,
            maxPagesToDisplay: maxPagesToDisplay,
            sortNames: sortKeys,
            fields: fields,
            loading: loading,
            displayFiltersFormatter: displayFiltersFormatter,
            debug: debug,
            onAcctionCalled: onAcctionCalled,
            filterValues: filterValues,
            sorts: sorts,
            columnVisibility: colsVisibility,
            onChangeSort: handleSortChange,
            resetSorts: resetSorts,
            headerTextProcessor: headerTextProcessor,
            clearFilters: handleClearFilters,
            textProcessor: textProcessor,
            actions: actions,
            gridId: id || 'grid',
            data: (data || []),
            loadingLabel: loadingLabel,
            allowSelect: allowSelect,
            onChangeFilter: onChangeFilter,
            viewFilters: viewFilters,
            openSettings: openSettings,
            toggleFilters: toggleFilters,
            filtersConfig: filtersConfig,
            meta: meta,
            toggleSettings: toggleSettings,
            handleChangePage: handleChangePage,
            toggleVisibility: toggleVisibility
        } },
        React__default['default'].createElement(GridRender, null)));
};

exports.default = SmartGrid;
//# sourceMappingURL=index.js.map
