import { InputPropsType } from '@alejandro.devop/bt-components';
import { IconType } from '@alejandro.devop/bt-types';
import fieldsMap from './fields-map';
import { ResolveUrlConfigType } from '@alejandro.devop/react-json-api-client';
export declare type GridContextType = {
    headers: GridHeadersType;
    clearFilters: () => void;
    headerNames: string[];
    filtersConfig: ({
        key: string;
    } & FilterOptionsType)[];
    onChangeRecordsPerpage: (n: number) => void;
    handleChangePage: (n: number) => void;
    onAcctionCalled?: (a: string, r: any) => void;
    applyGoToPage: (n: number) => void;
    selectedRecords: any[];
    sorts: GridSortsType;
    sortNames: string[];
    maxPagesToDisplay?: number;
    allowSelect?: boolean;
    onChangeSort: (key: string) => void;
    actions?: ActionType[];
    columnVisibility?: {
        [k: string]: boolean;
    };
    toggleVisibility: (k: string, b: boolean) => void;
    data: any[];
    loading?: boolean;
    loadingLabel?: string;
    gridId: string;
    toggleSettings: () => void;
    openSettings: boolean;
    displayFiltersFormatter?: (filters: any) => any;
    viewFilters?: boolean;
    filterValues: {
        [key: string]: any;
    };
    onChangeFilter: (k: string, v: any) => void;
    toggleFilters: () => void;
    headerTextProcessor?: (k: string) => string;
    textProcessor?: (k: string, r?: {
        key: string;
        value: any;
    }) => string;
    debug?: boolean;
    resetSorts: () => void;
    fields: {
        [k: string]: GridFieldType;
    };
    meta: {
        totalRecords: number;
        totalPages: number;
        currentPage: number;
        nextPage: number;
        prevPage: number;
        recordsPerPage: number;
    };
};
export declare type ActionType = {
    label: string;
    id: string;
    icon: IconType;
};
export declare type GridHeaderType = {
    label?: string;
    value?: ((data: any) => string) | string;
    sort?: boolean;
    hide?: boolean;
};
export declare type SortValueType = -1 | 0 | 1;
export declare type GridHeadersType = {
    [k: string]: GridHeaderType;
};
export declare type GridSortsType = {
    [k: string]: SortValueType;
};
export declare type UseGridHeadersConfigType = {
    fields: {
        [k: string]: GridFieldType;
    };
    columnVisibility?: {
        [k: string]: boolean;
    };
};
export declare type UseGridHeadersReturnType = {
    headerNames: string[];
    sortKeys: string[];
};
export declare type FilterOptionsType = {
    type?: keyof typeof fieldsMap;
    name?: string;
    label?: string;
    inputOptions?: InputPropsType;
    defaultValue?: {
        [k: string]: any;
    };
    preProcessFilterValue?: (f: any) => any;
    dropdownOptions?: {
        color?: 'primary' | 'default' | 'secondary' | 'success' | 'info' | 'warning' | 'danger';
        direction?: 'down' | 'left' | 'right' | 'up';
        options?: {
            label: string;
            value: string | number;
        }[];
    };
    selectOptions?: {
        ajax?: boolean;
        advanced?: boolean;
        columnsMapping?: {
            label: string;
            value: string;
            labelProcessor?: (label: string) => string;
        };
        empty?: string;
        multiple?: boolean;
        url?: string;
        disableGroup?: boolean;
        requestConfig?: ResolveUrlConfigType;
        value?: {
            [key: string]: any;
        };
    };
};
export declare type GridFieldType = {
    filter?: boolean;
    value?: ((data: any) => string) | string;
    label?: string;
    hideLabel?: boolean;
    sort?: boolean;
    notReadable?: boolean;
    filterOptions?: FilterOptionsType;
};
export declare type GridConfig<UrlType> = {
    debug?: boolean;
    id?: string;
    url: UrlType;
    columnVisibility?: {
        [k: string]: boolean;
    };
    defaultFilters?: {
        [key: string]: any;
    };
    fields: {
        [k: string]: GridFieldType;
    };
    filtersVisible?: boolean;
    loadingLabel?: string;
    allowSelect?: boolean;
    actions: ActionType[];
    maxPagesToDisplay?: number;
    displayFiltersFormatter?: (f: any) => void;
    headers: {
        [k: string]: GridHeaderType;
    };
};
export declare type StatesType = {
    recordsPerPage: number;
    recordsPerPageControl: number;
};
export declare type MetaDataType = {
    totalRecords: number;
    totalPages: number;
    currentPage: number;
    nextPage: number;
    prevPage: number;
    recordsPerPage: number;
};
export declare type HeaderExtraConfigType = {
    label?: string;
    hide?: boolean;
};
